function lines_out = displayResults(fname, N)

if (nargin == 1)
    N = 1;
end

lines_out = [];

fid = fopen([fname '-0.cycle']);
if (fid == -1)
    display(['Could not find ' fname '-0.cycle']);
    return;
end
fclose(fid);
trapezoids = load([fname '.trapezoids']);
lines = load([fname '.lines']);
img = imread([fname '.pgm']);
%img = imread(['../textures/' fname(length(fname)) '-canny.pgm']);

for i=1:N
    fid = fopen([fname '-' int2str(i-1) '.cycle'],'r');
    if (fid == -1)
        display(['Could not open file ' fname '-' int2str(i-1) '.cycle']);
        break
    end
    num = fscanf(fid, '%d',1);
    if (num == 0)
        display(['Empty cycle for ' fname '-' int2str(i-1) '.cycle']);
    end
    cycle = (fscanf(fid,'%d',[6,num]))';
    fclose(fid);
    if (size(cycle,1) == 0)
        break;
    end
    
    % display over image
    imshow(img);
    brighten(0.7);
%     axis off;
%     axis equal;
%     axis ij;
%     axis([1 96 1 96]);
     hold on;
    %display_lines('k-', lines);
    lines_out = display_cycle(cycle, trapezoids);
    print('-r0','-dpng', [fname '-cycle-' int2str(i-1)]);
    print('-depsc2', [fname '-cycle-' int2str(i-1)]);
     close;
    
    %display over detected lines
%     axis off;
%     axis equal;
%     axis ij;
%     axis([0 96 0 96]);
%     hold on;
%     display_lines('k-', lines);
%     %display_cycle(cycle, trapezoids);
%     print('-r0','-dpng', [fname '-lines-' int2str(i-1)]);
%     print('-depsc2', [fname '-lines-' int2str(i-1)]);
%     close;
    
end

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lines=display_cycle(cycle, axes)

lines = [];
count = 0;

for i=1:size(cycle,1)
    if (cycle(i,5) == 0) % dashed
        a = floor(cycle(i,1)/4) + 1;
        b = floor(cycle(i,2)/4) + 1;
        
        if (mod(cycle(i,1),4) < 2)
            A1 = axes(a,5:6);
            A2 = axes(a,7:8);
            cA = axes(a,9:10);
            P1 = axes(a,1:2);
        else
            A1 = axes(a,11:12);
            A2 = axes(a,9:10);
            cA = axes(a,7:8);
            P1 = axes(a,3:4);
        end
        if (mod(cycle(i,2),4) < 2)
            B1 = axes(b,5:6);
            B2 = axes(b,7:8);
            cB = axes(b,9:10);
            P2 = axes(b,1:2);
        else
            B1 = axes(b,11:12);
            B2 = axes(b,9:10);
            cB = axes(b,7:8);
            P2 = axes(b,3:4);
        end

        dir1 = mod(cycle(i,1),2); % 0 positive, 1 negative area
        dir1 = 2*(1-dir1)-1;
        dir2 = mod(cycle(i,2),2);
        dir2 = 2*(1-dir2)-1;
        
        if (dir1 == dir2)
            p = plot([A1(1) B2(1)], [A1(2) B2(2)], 'k-');
            set(p,'LineWidth',5);
            p = plot([A2(1) B1(1)], [A2(2) B1(2)], 'k-');
            set(p,'LineWidth',5);
            count = count + 2;
            lines(count-1,:) = [A1 B2];
            lines(count,:) = [A2 B1];
        else
            p = plot([A1(1) B1(1)], [A1(2) B1(2)], 'k-');
            set(p,'LineWidth',5);
            p = plot([A2(1) B2(1)], [A2(2) B2(2)], 'k-');
            set(p,'LineWidth',5);
            count = count + 2;
            lines(count-1,:) = [A1 B1];
            lines(count,:) = [A2 B2];
        end
         p = plot([P1(1) P2(1)], [P1(2) P2(2)], 'r--');
         set(p,'LineWidth',5);
                
    elseif (cycle(i,5) == 1) % solid
        j = floor(cycle(i,1)/4) + 1;
        p = plot(axes(j,[5,9]), axes(j,[6,10]), 'k-');
        set(p,'LineWidth',5);
        p = plot(axes(j,[7,11]), axes(j,[8,12]), 'k-');
        set(p,'LineWidth',5);
        count = count + 2;
        lines(count-1,:) = axes(j,[5 6 9 10]);
        lines(count,:) = axes(j,[7 8 11 12]);
         p = plot(axes(j,[1,3]), axes(j,[2,4]), 'r--');
         set(p,'LineWidth',5);
    else % auxiliary
        a = floor(cycle(i,1)/4) + 1;
        if (mod(cycle(i,1),4) < 2)
            p = plot(axes(a,[5,7]), axes(a,[6,8]),'k-');
            set(p,'LineWidth',5);
            count = count + 1;
            lines(count,:) = axes(a,5:8);
        else
            p = plot(axes(a,[9,11]), axes(a,[10,12]),'k-');
            set(p,'LineWidth',5);
            count = count + 1;
            lines(count,:) = axes(a,9:12);
        end
        b = floor(cycle(i,2)/4) + 1;
        if (mod(cycle(i,2),4) < 2)
            p = plot(axes(b,[5,7]), axes(b,[6,8]),'k-');
            set(p,'LineWidth',5);
            count = count + 1;
            lines(count,:) = axes(b,5:8);
        else
            p = plot(axes(b,[9,11]), axes(b,[10,12]),'k-');
            set(p,'LineWidth',5);
            count = count + 1;
            lines(count,:) = axes(b,9:12);
        end
    end
end


function d=find_dir(A,B,C)

d=(B(1)-A(1))*(C(2)-A(2)) - (C(1)-A(1))*(B(2)-A(2));

if (d < 0)
    d = -1;
else
    d = 1;
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function display_lines(color, data)
%
% FUNCTION display_lines(img, data)
%
% Displays the lines in data superimposed over the image img.
%
% Joachim Stahl
% January 18, 2005
%

% uncommment to draw lines over white background
%img = 255 + 0*img;

%imagesc(img);
% colormap(gray);
% axis off;
% axis equal;
% hold on;
% brighten(1);

%color = 'r-';
for i=1:size(data,1)
    if (data(i,5) == 0)
        plot([data(i,1),data(i,3)],[data(i,2),data(i,4)],color);
    end
end

return;

%%% END %%%
