#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>

using namespace std;

int sign(double);
double distance(double x1, double y1, double x2, double y2);
int seg_intersect(double, double, double, double, double, double, double, double);
double* lines_intersection(double,double,double,double,double,double,double,double);
double* triangle_incenter(double,double,double,double,double,double);
double* line_project_point(double,double,double,double,double,double);
double* segment_contains_point(double,double,double,double,double,double);
int find_dir(double,double,double,double,double,double);


double* single_trapezoid(double A1x,double A1y,double A2x,double A2y,double B1x,double B1y,double B2x,double B2y,double Ix,double Iy,int parallel) {

   double *trapezoid;
   double temp, *inctr, *prA1, *prA2, *prB1, *prB2, *uv;
   double P1[2], P2[2], PP1[2], PP2[2], pr1[2], pr2[2], d1, d2;
   int prflag = 1, found;

   if (parallel == 0) {
      if (distance(Ix,Iy,A1x,A1y) < distance(Ix,Iy,A2x,A2y)) {
         temp = A2x;
         A2x = A1x;
         A1x = temp;
         temp = A2y;
         A2y = A1y;
         A1y = temp;
      }
      if (distance(Ix,Iy,B1x,B1y) < distance(Ix,Iy,B2x,B2y)) {
         temp = B2x;
         B2x = B1x;
         B1x = temp;
         temp = B2y;
         B2y = B1y;
         B1y = temp;
      }
      inctr = triangle_incenter(A1x,A1y,B1x,B1y,Ix,Iy);
   } else {
      inctr = new double[2];
      inctr[0] = (A2x+B2x)/2;
      inctr[1] = (A2y+B2y)/2;
   }

   prA1 = line_project_point(Ix,Iy,inctr[0],inctr[1],A1x,A1y);
   prA2 = line_project_point(Ix,Iy,inctr[0],inctr[1],A2x,A2y);
   prB1 = line_project_point(Ix,Iy,inctr[0],inctr[1],B1x,B1y);
   prB2 = line_project_point(Ix,Iy,inctr[0],inctr[1],B2x,B2y);

   delete[] inctr;

   uv = segment_contains_point(prA1[0],prA1[1],prA2[0],prA2[1],prB1[0],prB1[1]);
   if ( (uv[0] >= -0.0001) && (uv[0] <= 1.0001) && (fabs(uv[1]) < 0.0001) )
      prflag *= 5;
   delete[] uv;

   uv = segment_contains_point(prA1[0],prA1[1],prA2[0],prA2[1],prB2[0],prB2[1]);
   if ( (uv[0] >= -0.0001) && (uv[0] <= 1.0001) && (fabs(uv[1]) < 0.0001) )
      prflag *= 7;
   delete[] uv;

   uv = segment_contains_point(prB1[0],prB1[1],prB2[0],prB2[1],prA1[0],prA1[1]);
   if ( (uv[0] >= -0.0001) && (uv[0] <= 1.0001) && (fabs(uv[1]) < 0.0001) )
      prflag *= 2;
   delete[] uv;

   uv = segment_contains_point(prB1[0],prB1[1],prB2[0],prB2[1],prA2[0],prA2[1]);
   if ( (uv[0] >= -0.0001) && (uv[0] <= 1.0001) && (fabs(uv[1]) < 0.0001) )
      prflag *= 3;
   delete[] uv;

   if ( (prflag < 3) || (prflag == 4) || (prflag == 8) ) return 0;

   found = 0; 
   if (prflag % 2 == 0) {
      pr1[0] = prA1[0];
      pr1[1] = prA1[1];
      P1[0] = A1x;
      P1[1] = A1y;
      d1 = prA1[2];
      found = 1;
   }
   delete[] prA1;
   if (prflag % 3 == 0) {
      if (found == 0) {
         pr1[0] = prA2[0];
         pr1[1] = prA2[1];
         P1[0] = A2x;
         P1[1] = A2y;
         d1 = prA2[2];
         found = 1;
      } else {
         pr2[0] = prA2[0];
         pr2[1] = prA2[1];
         P2[0] = A2x;
         P2[1] = A2y;
         d2 = prA2[2];
         found = 2;
      }
   }
   delete[] prA2;
   if (prflag % 5 == 0) {
      if (found == 0) {
         pr1[0] = prB1[0];
         pr1[1] = prB1[1];
         P1[0] = B1x;
         P1[1] = B1y;
         d1 = prB1[2];
         found = 1;
      } else if (found == 1) {
         pr2[0] = prB1[0];
         pr2[1] = prB1[1];
         P2[0] = B1x;
         P2[1] = B1y;
         d2 = prB1[2];
         found = 2;
      }
   }
   delete[] prB1;
   if (prflag % 7 == 0) {
      if (found == 0) {
         pr1[0] = prB2[0];
         pr1[1] = prB2[1];
         P1[0] = B2x;
         P1[1] = B2y;
         d1 = prB2[2];
         found = 1;
      } else if (found == 1) {
         pr2[0] = prB2[0];
         pr2[1] = prB2[1];
         P2[0] = B2x;
         P2[1] = B2y;
         d2 = prB2[2];
         found = 2;
      }
   }
   delete[] prB2;

   PP1[0] = 2*pr1[0] - P1[0];
   PP1[1] = 2*pr1[1] - P1[1];
   PP2[0] = 2*pr2[0] - P2[0];
   PP2[1] = 2*pr2[1] - P2[1];
   if (seg_intersect(P1[0],P1[1],P2[0],P2[1],PP1[0],PP1[1],PP2[0],PP2[1]) == 1) {
      temp = PP1[0];
      PP1[0] = P1[0];
      P1[0] = temp;
      temp = PP1[1];
      PP1[1] = P1[1];
      P1[1] = temp;
   }

   if (find_dir(P2[0],P2[1],P1[0],P1[1],PP1[0],PP1[1]) < 0) {
      temp = P1[0];
      P1[0] = P2[0];
      P2[0] = temp;
      temp = P1[1];
      P1[1] = P2[1];
      P2[1] = temp;

      temp = PP1[0];
      PP1[0] = PP2[0];
      PP2[0] = temp;
      temp = PP1[1];
      PP1[1] = PP2[1];
      PP2[1] = temp;

      temp = pr1[0];
      pr1[0] = pr2[0];
      pr2[0] = temp;
      temp = pr1[1];
      pr1[1] = pr2[1];
      pr2[1] = temp;
   }   

   trapezoid = new double[16];

   trapezoid[0] = pr1[0];
   trapezoid[1] = pr1[1];
   trapezoid[2] = pr2[0];
   trapezoid[3] = pr2[1];
   trapezoid[4] = P1[0];
   trapezoid[5] = P1[1];
   trapezoid[6] = PP1[0];
   trapezoid[7] = PP1[1];
   trapezoid[8] = P2[0];
   trapezoid[9] = P2[1];
   trapezoid[10] = PP2[0];
   trapezoid[11] = PP2[1];
   trapezoid[12] = 0;
   trapezoid[13] = 0;
   trapezoid[14] = fabs(distance(pr1[0],pr1[1],pr2[0],pr2[1])*(d1+d2)); //area
   trapezoid[15] = fabs(distance(P1[0], P1[1], P2[0], P2[1])); // possible gap

   return trapezoid;
}

vector<double*>* find_trapezoids(double A1x,double A1y,double A2x,double A2y,double B1x,double B1y,double B2x,double B2y) {

   vector<double*> *trapezoids = new vector<double*>;
   double *trap, B3;
   double *I = lines_intersection(A1x,A1y,A2x,A2y,B1x,B1y,B2x,B2y);

   if ( ((int)(I[0]) == 1) && (distance(I[1],I[2],A1x,A1y) > 10000) )
      I[0] = 0;

   if (I[0] == 2) return trapezoids;

   else if (I[0] == 1) {
      double tmp1 = ( (A1x-I[1])*(A2x-I[1]) + (A1y-I[2])*(A2y-I[2]) );
      double tmp2 = ( (B1x-I[1])*(B2x-I[1]) + (B1y-I[2])*(B2y-I[2]) );
      if (tmp1 < -1) {
         trap = single_trapezoid(A1x,A1y,I[1],I[2],B1x,B1y,B2x,B2y,I[1],I[2],0);
         if (trap != 0)
            (*trapezoids).push_back(trap);
         trap = single_trapezoid(A2x,A2y,I[1],I[2],B1x,B1y,B2x,B2y,I[1],I[2],0);
         if (trap != 0)
            (*trapezoids).push_back(trap);

      } else if (tmp2 < -1) {
         trap = single_trapezoid(A1x,A1y,A2x,A2y,B1x,B1y,I[1],I[2],I[1],I[2],0);
         if (trap != 0)
            (*trapezoids).push_back(trap);
         trap = single_trapezoid(A1x,A1y,A2x,A2y,I[1],I[2],B2x,B2y,I[1],I[2],0);
         if (trap != 0)
            (*trapezoids).push_back(trap);

      } else {
         trap = single_trapezoid(A1x,A1y,A2x,A2y,B1x,B1y,B2x,B2y,I[1],I[2],0);
         if (trap != 0)
            (*trapezoids).push_back(trap);

      }
   } else { // I[0] == 0
      if (seg_intersect(A1x,A1y,B1x,B1y,A2x,A2y,B2x,B2y)) {
         B3 = B2x;
         B2x = B1x;
         B1x = B3;
         B3 = B2y;
         B2y = B1y;
         B1y = B3;
      }
      I[1] = (A1x+B1x)/2;
      I[2] = (A1y+B1y)/2;
      trap = single_trapezoid(A1x,A1y,A2x,A2y,B1x,B1y,B2x,B2y,I[1],I[2],1);
      if (trap != 0)
         (*trapezoids).push_back(trap);
   }

   delete[] I;

   return trapezoids;
}

vector<double*>* construct_trapezoids(vector<double*> *lines) {

   vector<double*> *answer = new vector<double*>;
   vector<double*> *tmp;
   double *trapezoid;
   double *line1, *line2;
   double A1x,A1y,A2x,A2y,Agap,B1x,B1y,B2x,B2y,Bgap;

   for (int i=0; i<(int)(lines->size()-1); i++) {
      line1 = (*lines)[i];
      A1x = line1[0];
      A1y = line1[1];
      A2x = line1[2];
      A2y = line1[3];
      Agap = line1[4];
      if (Agap > 0) break;
      for (int j=i+1; j<(int)(lines->size()); j++) {
         line2 = (*lines)[j];
         B1x = line2[0];
         B1y = line2[1];
         B2x = line2[2];
         B2y = line2[3];
         Bgap = line2[4];
         //if (Bgap > 0) break;
         tmp = find_trapezoids(A1x,A1y,A2x,A2y,B1x,B1y,B2x,B2y);
         if (tmp != 0) {
         for (int k=0; k<(int)(*tmp).size(); k++) {
            trapezoid = (*tmp)[k];
            trapezoid[12] = i;
            trapezoid[13] = j;
            if (trapezoid[14] > trapezoid[15]) {
               if (Bgap < 1)
                  trapezoid[15] = 0;
               answer->push_back(trapezoid);
            }
         }
         (*tmp).clear();
         delete tmp;
         }
      }

      
   }

   return answer;
}

/*int main(int argc, char** argv) {

   ifstream in(argv[1]);
   int line_count = atoi(argv[2]);

   vector<double*> lines;
   vector<double*> trapezoids;
   double *one_line;
   double *T;

   for (int i=0; i<line_count; i++) {
       one_line = new double[4];
       in >> one_line[0] >> one_line[1] >> one_line[2] >> one_line[3];
       lines.push_back(one_line);      
   }

   in.close();
cout << "pre\n";
   trapezoids = construct_trapezoids(lines);
cout << "post\n";
   ofstream out("trapezoids.tbl");
cout << "fileopen\n";
   for (int i=0; i<trapezoids.size(); i++) {
      T = trapezoids[i];
      for (int k=0; k<15; k++)
         out << T[k] << " ";
      out << endl;

   }

   out.close();

   return 0;
}*/




double distance(double x1, double y1, double x2, double y2) {
   return sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) );
}

int seg_intersect(double Ax, double Ay, double Bx, double By, double Cx, double Cy, double Dx, double Dy){
   double test1, test2;
   /*double *uv1, *uv2;
   double *I = lines_intersection(Ax,Ay,Bx,By,Cx,Cy,Dx,Dy);
   uv1 = segment_contains_point(Ax,Ay,Bx,By,I[0],I[1]);
   uv2 = segment_contains_point(Cx,Cy,Dx,Dy,I[0],I[1]);

   if ( (uv1[0] >= -0.0001) && (uv1[0] <= 1.0001) && (fabs(uv1[1]) < 0.0001) &&
        (uv2[0] >= -0.0001) && (uv2[0] <= 1.0001) && (fabs(uv2[1]) < 0.0001) )
      return 1;
   else
      return 0;*/

   test1 = ((Ay-Cy)*(Dx-Cx) - (Dy-Cy)*(Ax-Cx))*((By-Cy)*(Dx-Cx) - (Dy-Cy)*(Bx-Cx));
   test2 = ((Cy-Ay)*(Bx-Ax) - (By-Ay)*(Cx-Ax))*((Dy-Ay)*(Bx-Ax) - (By-Ay)*(Dx-Ax));
   if (test1 < 0 && test2 < 0) return 1;
   else return 0;
    
}

double* lines_intersection(double A1x,double A1y,double A2x,double A2y,double B1x,double B1y,double B2x,double B2y) {

   double *I = new double[3];

   double mA, mB, cA, cB;

   if (A1x == A2x) {
      if (B1x == B2x) {
         if (A1x == B1x) {
            I[0] = 2; I[1] = 0; I[2] = 0; return I;
         } else {
            I[0] = 0; I[1] = 0; I[2] = 0; return I;
         }
      } else {
         mB = (B1y - B2y)/(B1x - B2x);
         cB = B1y - mB*B1x;
         I[0] = 1; I[1] = A1x; I[2] = mB*A1x + cB;
         return I;
      }
   } else {
      mA = (A1y - A2y)/(A1x - A2x);
      cA = A1y - mA*A1x;
      if (B1x == B2x) {
         I[0] = 1; I[1] = B1x; I[2] = mA*B1x + cA;
         return I;
      } else {
         mB = (B1y - B2y)/(B1x - B2x);
         cB = B1y - mB*B1x;
         if (mA == mB) {
            if (cA == cB) {
               I[0] = 2; I[1] = 0; I[2] = 0; return I;
            } else {
               I[0] = 0; I[1] = 0; I[2] = 0; return I;
            }
         } else {
            I[0] = 1;
            I[1] = (cB - cA)/(mA - mB);
            I[2] = mA*I[1] + cA;
            return I;
         }
      }
   }
}

double* triangle_incenter(double Ax,double Ay,double Bx,double By,double Cx,double Cy) {

   double a,b,c, perimeter;
   double *center = new double[2];

   a = distance(Ax,Ay,Bx,By);
   b = distance(Bx,By,Cx,Cy);
   c = distance(Cx,Cy,Ax,Ay);

   perimeter = a+b+c;

   if ( perimeter == 0) {
      center[0] = Ax;
      center[1] = Ay;
   } else {
      center[0] = (b*Ax + c*Bx + a*Cx)/perimeter;
      center[1] = (b*Ay + c*By + a*Cy)/perimeter;
   }

   return center;
}

double* line_project_point(double P1x,double P1y,double P2x,double P2y,double Px,double Py) {

   double *pnd = new double[3];
   double bot,t;

   bot = (P2x-P1x)*(P2x-P1x) + (P2y-P1y)*(P2y-P1y);
   t = ( (P1x-Px)*(P1x-P2x) + (P1y-Py)*(P1y-P2y) ) / bot;
   pnd[0] = P1x + t*(P2x-P1x);
   pnd[1] = P1y + t*(P2y-P1y);
   pnd[2] = distance(Px,Py,pnd[0],pnd[1]);

   return pnd;
}

double* segment_contains_point(double P1x,double P1y,double P2x,double P2y,double Px,double Py) {

   double *uv = new double[2];
   double normsq;

   normsq = (P2x-P1x)*(P2x-P1x) + (P2y-P1y)*(P2y-P1y);

   if (normsq == 0) {
      if ( (Px == P1x) && (Py == P1y) ) {
         uv[0] = 0.5;
         uv[1] = 0.0;
      } else {
         uv[0] = 0.5;
         uv[1] = 999999.99; // +inf
      }
   } else {

      uv[0] = ( (Px-P1x)*(P2x-P1x) + (Py-P1y)*(P2y-P1y) ) / normsq;
      uv[1] = sqrt( ((uv[0]-1)*P1x - uv[0]*P2x + Px)*((uv[0]-1)*P1x - uv[0]*P2x + Px) + ((uv[0]-1)*P1y - uv[0]*P2y + Py)*((uv[0]-1)*P1y - uv[0]*P2y + Py) ) / sqrt(normsq);
   }

   return uv;

}

int sign(double d) {
   if (d == 0) return 0;
   return ((d>0)?1:-1);
}

int find_dir(double Ax,double Ay,double Bx,double By,double Cx,double Cy) {

   return sign( (Bx-Ax)*(Cy-Ay) - (Cx-Ax)*(By-Ay) );

}


