#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>
#include <string>

using namespace std;

vector<double*>* construct_trapezoids(vector<double*> *lines);
vector<double*>* construct_graph(vector<double*> *Ttable, vector<double*> *Ltable, double LAMBDA, int MAX_ENDPOINT);
int RC(vector<double*> *graph, int ncount, int ecount, int NumIteration, string fname);

int main(int argc, char** argv) {

   double LAMBDA;
   int N, MAX_ENDPOINT;

   if (argc < 2 || argc > 5) {
      cerr << "Usage: SRC foo [lambda [n e]]\n\tfoo    - image name (without extension)\n\tlambda - real value, default 1 (see cost function)\n\tn      - number of boundaries, default 1\n\te      - maximum endpoint proximity, default 20\n";
      return 1;
   }

   string fname = argv[1];
   ifstream in((fname+".lines").c_str());
   if (argc > 2)
      LAMBDA = atof(argv[2]);
   else
      LAMBDA = 1;
   if (argc > 3)
      N = atoi(argv[3]);
   else
      N = 1;
   if (argc == 5)
      MAX_ENDPOINT = atoi(argv[4]);
   else
      MAX_ENDPOINT = 20;
   
   

   vector<double*> *lines = new vector<double*>;
   vector<double*> *trapezoids;
   vector<double*> *graph;
   double *one_line, MAX_AREA, MAX_W1;
   double *T, *dataout;

   while(in.good()) {
       one_line = new double[5];
       in >> one_line[0] >> one_line[1] >> one_line[2] >> one_line[3] >> one_line[4];
       if (in.eof()) { delete[] one_line; break;}
       lines->push_back(one_line);      
   }
   cout << lines->size() << " lines read\n";
   in.close();

/*//DEBUG-BEGIN
for (int i=0; i<(int)lines->size(); i++) {
one_line = (*lines)[i];
cout << one_line[0] << ' ' << one_line[1] << ' ' << one_line[2] << ' ' << one_line[3] << ' ' << one_line[4] <<endl;
}
//DEBUG-END*/

   trapezoids = construct_trapezoids(lines);
   cout << "Trapezoids: " << 2*trapezoids->size() << endl;

   ofstream out((fname+".trapezoids").c_str());
   for (int i=0; i<(int)(trapezoids->size()); i++) {
      T = (*trapezoids)[i];
      for (int k=0; k<16; k++)
         out << T[k] << " ";
      out << endl;

   }
   out.close();

   graph = construct_graph(trapezoids, lines, LAMBDA, MAX_ENDPOINT);

   dataout = (*graph)[(*graph).size()-1];
   MAX_AREA = dataout[0];
   MAX_W1 = dataout[1];

   
   (*graph).pop_back();
   delete[] dataout;

   for (int i = 0; i < (int)(*graph).size(); i++) {
      dataout = (*graph)[i];
      dataout[2] = dataout[2]*600.0/MAX_AREA;
      dataout[3] = dataout[3]*600.0/MAX_W1;
//cout << dataout[0] << " " << dataout[1] << " " << dataout[2] << " " << dataout[3] << " " << dataout[4] << endl;
   }
//cerr << "done\n";

   RC(graph, (4*(trapezoids->size())), (graph->size()), N, fname);

   for (int i=0; i<(int)(graph->size()); i++) {
      dataout = (*graph)[i];
      delete[] dataout;
   }
   graph->clear();
   delete graph;

   for (int i=0; i<(int)(trapezoids->size()); i++) {
      dataout = (*trapezoids)[i];
      delete[] dataout;
   }
   trapezoids->clear();
   delete trapezoids;

   for (int i=0; i<(int)(lines->size()); i++) {
      dataout = (*lines)[i];
      delete[] dataout;
   }
   lines->clear();
   delete lines;




   return 0;
}

